@@ -1,24 +1,13 @@
-/********************************************************************
- KWin - the KDE window manager
- This file is part of the KDE project.
+/*
+    KWin - the KDE window manager
+    This file is part of the KDE project.
 
-Copyright (c) 2011 Lionel Chauvin <megabigbug@yahoo.fr>
-Copyright (c) 2011,2012 Cédric Bellegarde <gnumdk@gmail.com>
-Copyright (C) 2013 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2011 Lionel Chauvin <megabigbug@yahoo.fr>
+    SPDX-FileCopyrightText: 2011, 2012 Cédric Bellegarde <gnumdk@gmail.com>
+    SPDX-FileCopyrightText: 2013 Martin Gräßlin <mgraesslin@kde.org>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #include "appmenu.h"
 #include "x11client.h"
 #include "workspace.h"
@@ -30,7 +19,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "decorations/decorationbridge.h"
 #include <KDecoration2/DecorationSettings>
 
-using namespace KWin;
+namespace KWin
+{
 
 KWIN_SINGLETON_FACTORY(ApplicationMenu)
 
@@ -129,3 +119,5 @@ AbstractClient *ApplicationMenu::findAbstractClientWithApplicationMenu(const QSt
         && c->applicationMenuObjectPath() == menuObjectPath.path();
     });
 }
+
+} // namespace KWin
