@@ -555,7 +555,7 @@ void SceneOpenGL::insertWait()
  * Render cursor texture in case hardware cursor is disabled.
  * Useful for screen recording apps or backends that can't do planes.
  */
-void SceneOpenGL2::paintCursor()
+void SceneOpenGL2::paintCursor(const QRegion &rendered)
 {
     Cursor* cursor = Cursors::self()->currentCursor();
 
@@ -566,6 +566,17 @@ void SceneOpenGL2::paintCursor()
         return;
     }
 
+    // figure out which part of the cursor needs to be repainted
+    const QPoint cursorPos = cursor->pos() - cursor->hotspot();
+    const QRect cursorRect = cursor->rect();
+    QRegion region;
+    for (const QRect &rect : rendered) {
+        region |= rect.translated(-cursorPos).intersected(cursorRect);
+    }
+    if (region.isEmpty()) {
+        return;
+    }
+
     // lazy init texture cursor only in case we need software rendering
     if (!m_cursorTexture) {
         auto updateCursorTexture = [this] {
@@ -585,8 +596,6 @@ void SceneOpenGL2::paintCursor()
     }
 
     // get cursor position in projection coordinates
-    const QPoint cursorPos = cursor->pos() - cursor->hotspot();
-    const QRect cursorRect(0, 0, m_cursorTexture->width(), m_cursorTexture->height());
     QMatrix4x4 mvp = m_projectionMatrix;
     mvp.translate(cursorPos.x(), cursorPos.y());
 
@@ -598,11 +607,8 @@ void SceneOpenGL2::paintCursor()
     m_cursorTexture->bind();
     ShaderBinder binder(ShaderTrait::MapTexture);
     binder.shader()->setUniform(GLShader::ModelViewProjectionMatrix, mvp);
-    m_cursorTexture->render(QRegion(cursorRect), cursorRect);
+    m_cursorTexture->render(region, cursorRect);
     m_cursorTexture->unbind();
-
-    cursor->markAsRendered();
-
     glDisable(GL_BLEND);
 }
 
@@ -647,7 +653,7 @@ qint64 SceneOpenGL::paint(const QRegion &damage, const QList<Toplevel *> &toplev
             updateProjectionMatrix();
 
             paintScreen(&mask, damage.intersected(geo), repaint, &update, &valid, projectionMatrix(), geo, scaling);   // call generic implementation
-            paintCursor();
+            paintCursor(valid);
 
             GLVertexBuffer::streamingBuffer()->endOfFrame();
 
